/* 
 * File: Timer.js
 * Defines timer (based on requestAnimationFrame()
 *    Not using HTML/JS timer because of:
 *    http://stackoverflow.com/questions/13935262/settimeout-or-setinterval-or-requestanimationframe
 *    
 */

/*jslint node: true, vars: true, bitwise: true */
/*global angular */
/* find out more about jslint: http://www.jslint.com/help.html */

"use strict";

var gTimerService = function (who) { 
    who.timerService(); };

// Defines a timer class
function CSS450Timer(interval, callBack) {
    // interval: how often should the timer fire: in ms
    // callBack: when fired, will call this 

    this.mUpdateTime = interval;  // 
    this.mCallback = callBack;

    this.mCurrentTime = 0;
    this.mPreviousTime = 0;
    this.mElapsedTime = 0;
    this.mTimerIsRunning = false;
}

CSS450Timer.prototype.timerService = function () {
    if (this.mTimerIsRunning)
        requestAnimationFrame( this.timerService.bind(this) );
        // When system's requestAnimationFrame() wakes up, it will not know who is "this"
        // so we cannot simply pass in "this.timerService". By calling the Javascript
        // bind() function, we define a new funciton that is bound to the current object
        //  http://stackoverflow.com/questions/15455009/javascript-call-apply-vs-bind
        //
    this.mCurrentTime = Date.now();
    this.mElapsedTime += this.mCurrentTime - this.mPreviousTime;
    this.mPreviousTime = this.mCurrentTime;
    while (this.mElapsedTime > this.mUpdateTime) {
        this.mCallback(); 
        this.mElapsedTime -= this.mUpdateTime;
    }
};

CSS450Timer.prototype.start = function () {
    this.mTimerIsRunning = true;
    this.mCurrentTime = Date.now();
    this.mPreviousTime = this.mCurrentTime;
    this.mElapsedTime = 0;
f();
    this.timerService.call(this);
};

CSS450Timer.prototype.stop = function () {
    this.mTimerIsRunning = false;
    this.mElapsedTime = 0;
};

CSS450Timer.prototype.setUpdateInterval = function (interval) {
    this.mUpdateTime= interval;
};


