/* 
 * File: CSS450Timer.js
 * A timer utility   
 */

/*jslint node: true, vars: true, bitwise: true */
/*global angular */
/* find out more about jslint: http://www.jslint.com/help.html */

"use strict";

angular.module("CSS450Timer", [])
    .directive("cssTimer", function () {
        return {

            template: "Timer:[{{mEcho}}] [{{mInterval}}] [{{mCount}}] [{{mTimerIsOn}}]",

            scope: {
                mInterval: "=interval",
                mCallback: "=callback",
                mCount: "=count",
                mTimerIsOn: "=timeron"
            },

            controller: function ($scope, $interval) {
                $scope.mRun = undefined;
                $scope.mEcho = "OFF";

                // set defaults
                if (!angular.isDefined($scope.mInterval))
                    $scope.mInterval = 100;
                
                if (!angular.isDefined($scope.mCount))
                    $scope.mCount = 0;
                
                if (!angular.isDefined($scope.mTimerIsOn))
                    $scope.mTimerIsOn = false;
                
                // Default service function!
                $scope.timerService = function () {
                    $scope.mCount = $scope.mCount + 1;
                    $scope.mEcho = "ON:" + $scope.mCount;
                };
                
                if (!angular.isDefined($scope.mCallback)) 
                    $scope.mCallback = $scope.timerService;               
                
                $scope.start = function () {
                    if (angular.isDefined($scope.mRun))
                        return;
                    $scope.mEcho = "ON";          
                    $scope.mRun = $interval(
                        $scope.mCallback, 
                        $scope.mInterval);
                };
                
                $scope.stop = function() {
                    if (angular.isDefined($scope.mRun)) {
                        $interval.cancel($scope.mRun);
                        $scope.mRun = undefined;
                        $scope.mEcho = "OFF"; 
                    }
                    
                };
                
                $scope.$on("timer-stopped", function () {
                    if (!($scope.mTimerIsOn))
                        $scope.stop();
                });
                $scope.$on("timer-started", function () {
                    if (($scope.mTimerIsOn))
                        $scope.start();
                });
                
                if ($scope.mTimerIsOn)
                    $scope.start();
            }
        };
    });